@@ -50,12 +50,24 @@ fileGetter <- function(roots, restrictions, filetypes, pattern, hidden = FALSE)
   }
 
   function(dir, root) {
-    currentRoots <- if (class(roots) == "function") roots() else roots
+    currentRoots <- if (inherits(roots, "function")) roots() else roots
 
     if (is.null(names(currentRoots))) stop("Roots must be a named vector or a function returning one")
     if (is.null(root)) root <- names(currentRoots)[1]
     
     fulldir <- path(currentRoots[root], paste0(dir, collapse = "/"))
+    selectedFile = ""
+    if(file.exists(fulldir) && !dir.exists(fulldir)){
+      #dir is a normal file, not a directory
+      #get the filename, and use it as the selectedFile
+      selectedFile = sub(".*/(.*)$","\\1",fulldir)
+      #shorten the directory
+      fulldir = sub("(.*)/.*$","\\1",fulldir)
+      #dir also needs shortened for breadcrumps
+      dir = sub("(.*)/.*$","\\1",dir)
+    }
+    
+    
     writable <- as.logical(file_access(fulldir, "write"))
     files <- suppressWarnings(dir_ls(fulldir, all = hidden, fail = FALSE))
   
@@ -100,7 +112,8 @@ fileGetter <- function(roots, restrictions, filetypes, pattern, hidden = FALSE)
       exist = as.logical(file_exists(fulldir)),
       breadcrumps = I(c("", breadcrumps[breadcrumps != ""])),
       roots = I(names(currentRoots)),
-      root = root
+      root = root,
+      selectedFile = selectedFile
     )
   }
 }
@@ -361,6 +374,8 @@ shinyFileChoose <- function(input, id, updateFreq = 0, session = getSession(),
 #' @param icon An optional \href{http://shiny.rstudio.com/reference/shiny/latest/icon.html}{icon} to appear on the button.
 #' 
 #' @param style Additional styling added to the button (e.g., "margin-top: 25px;")
+#' 
+#' @param viewtype View type to use in the file browser. One of "detail" (default), "list", or "icon"
 #'
 #' @param filetype A named list of file extensions. The name of each element
 #' gives the name of the filetype and the content of the element the possible
@@ -382,8 +397,9 @@ shinyFileChoose <- function(input, id, updateFreq = 0, session = getSession(),
 #'
 #' @export
 #'
-shinyFilesButton <- function(id, label, title, multiple, buttonType="default", class=NULL, icon=NULL, style=NULL) {
+shinyFilesButton <- function(id, label, title, multiple, buttonType="default", class=NULL, icon=NULL, style=NULL, viewtype="detail") {
   value <- restoreInput(id = id, default = NULL)
+  viewtype <- if (length(viewtype) > 0 && viewtype %in% c("detail", "list", "icon")) viewtype else "detail"
   tagList(
     singleton(tags$head(
       tags$script(src = "sF/shinyFiles.js"),
@@ -406,6 +422,7 @@ shinyFilesButton <- function(id, label, title, multiple, buttonType="default", c
       "data-title" = title,
       "data-selecttype" = ifelse(multiple, "multiple", "single"),
       "data-val" = value,
+      "data-view" = paste0("sF-btn-", viewtype),
       list(icon, label)
     )
   )
@@ -418,8 +435,9 @@ shinyFilesButton <- function(id, label, title, multiple, buttonType="default", c
 #'
 #' @export
 #'
-shinyFilesLink <- function(id, label, title, multiple, class=NULL, icon=NULL, style=NULL) {
+shinyFilesLink <- function(id, label, title, multiple, class=NULL, icon=NULL, style=NULL, viewtype="detail") {
   value <- restoreInput(id = id, default = NULL)
+  viewtype <- if (length(viewtype) > 0 && viewtype %in% c("detail", "list", "icon")) viewtype else "detail"
   tagList(
     singleton(tags$head(
       tags$script(src = "sF/shinyFiles.js"),
@@ -442,6 +460,7 @@ shinyFilesLink <- function(id, label, title, multiple, class=NULL, icon=NULL, st
       "data-title" = title,
       "data-selecttype" = ifelse(multiple, "multiple", "single"),
       "data-val" = value,
+      "data-view" = paste0("sF-btn-", viewtype),
       list(icon, label)
     )
   )
@@ -503,7 +522,7 @@ shinyFilesLink <- function(id, label, title, multiple, class=NULL, icon=NULL, st
 #' @export
 #'
 parseFilePaths <- function(roots, selection) {
-  roots <- if (class(roots) == "function") roots() else roots
+  roots <- if (inherits(roots, "function")) roots() else roots
   
   if (is.null(selection) || is.na(selection) || is.integer(selection) || length(selection$files) == 0) {
     tibble(
