@@ -428,7 +428,7 @@ var shinyFiles = (function() {
             do {
               nSib = nextSibling(parDir);
               parDir = parentDir(parDir);
-            } while (nSib.length === 0);
+            } while (nSib.length === 0 && parDir.length > 0);
 
             newElement = nSib;
           } else {
@@ -555,18 +555,14 @@ var shinyFiles = (function() {
     var parsedFiles = {};
     data.files.filename.forEach(function(d, i) {
       try{
-        var mTime = data.files.mtime[i].split('-');
-        var cTime = data.files.ctime[i].split('-');
-        var aTime = data.files.atime[i].split('-');
-        // month index starts as zero
         parsedFiles[d] = {
           name: d,
           extension: data.files.extension[i],
           isDir: data.files.isdir[i],
           size: data.files.size[i],
-          mTime: new Date(mTime[0], mTime[1]-1, mTime[2], mTime[3], mTime[4]),
-          cTime: new Date(cTime[0], cTime[1]-1, cTime[2], cTime[3], cTime[4]),
-          aTime: new Date(aTime[0], aTime[1]-1, aTime[2], aTime[3], aTime[4])
+          mTime: new Date(data.files.mtime[i]),
+          cTime: new Date(data.files.ctime[i]),
+          aTime: new Date(data.files.atime[i])
         };
       } catch(err) {
         //This can happen if there is a broken link, for example
@@ -577,8 +573,10 @@ var shinyFiles = (function() {
       files: parsedFiles,
       location: data.breadcrumps,
       writable: data.writable,
+      exist: data.exist,
       rootNames: data.roots,
-      selectedRoot: data.root
+      selectedRoot: data.root,
+      selectedFile: data.selectedFile
     };
   };
     
@@ -750,25 +748,45 @@ var shinyFiles = (function() {
     removeFileChooser(button, modal, data);
   };
     
-    var setPermission = function(modal, writable) {
-      var currentState = $(modal).find('.sF-permission').length == 0;
-      var footer = $(modal).find('.modal-footer');
-      var overwrite = $($(modal).data('button')).hasClass('shinySave') ? true : filesSelected(modal);
-      
-      $(modal).find('#sF-btn-newDir').prop('disabled', !(overwrite && writable));
-      
-      if(writable || !overwrite) {
-        footer.find('.sF-permission').remove();
-      } else if(currentState != writable) {
-        footer.prepend(
-          $('<div>').addClass('sF-permission text-warning').append(
-            $('<span>').addClass('glyphicon glyphicon-warning-sign')
-          ).append(
-            $('<span>').text('No write permission for folder')
-          )
+  var setPermission = function(modal, writable) {
+    var currentState = $(modal).find('.sF-warning').length == 0;
+    var footer = $(modal).find('.modal-footer');
+    var overwrite = $($(modal).data('button')).hasClass('shinySave') ? true : filesSelected(modal);
+    
+    $(modal).find('#sF-btn-newDir').prop('disabled', !(overwrite && writable));
+    
+    if(writable || !overwrite) {
+      footer.find('.sF-warning').remove();
+    } else if(currentState != writable) {
+      footer.prepend(
+        $('<div>').addClass('sF-warning text-warning').append(
+          $('<span>').addClass('glyphicon glyphicon-warning-sign')
+        ).append(
+          $('<span>').text('No write permission for folder')
         )
-      }
+      )
     }
+  }
+  
+  var setExists = function(modal, exists) {
+    var currentState = $(modal).find('.sF-warning').length == 0;
+    var footer = $(modal).find('.modal-footer');
+    
+    $(modal).find('#sF-btn-newDir').prop('disabled', !(exists));
+    
+    
+    footer.find('.sF-warning').remove();
+    if(!exists) {
+      footer.prepend(
+        $('<div>').addClass('sF-warning text-warning').append(
+          $('<span>').addClass('glyphicon glyphicon-warning-sign')
+        ).append(
+          $('<span>').text('Folder does not exist')
+        )
+      )
+    }
+  }
+  
   // General functionality ends
     
     
@@ -782,7 +800,7 @@ var shinyFiles = (function() {
     
     // Creating modal
     var modal = $('<div>', {id: $(button).attr('id')+'-modal'}).addClass('sF-modalContainer modal fade').css('display', 'block').append(
-      $('<div>').addClass('sF-modal modal-dialog').append(
+      $('<div>').addClass('sF-modal modal-dialog modal-lg').append(
         $('<div>').addClass('modal-content').append(
           $('<div>').addClass('modal-header').append(
             $('<button>', {html: '&times;', type: 'button'}).addClass('close')
@@ -853,6 +871,26 @@ var shinyFiles = (function() {
                   )
                 )
               )
+            ).append(
+              $('<div>').addClass('sF-textChoice btn-group input-group-btn btn-group-sm').append(
+                $('<button>', {id: 'sF-btn-textChoice', text: ''}).addClass('btn btn-default dropdown-toggle').prepend(
+                  $('<span>').addClass('glyphicon glyphicon-pencil')
+                )
+              ).append(
+                $('<ul>').addClass('dropdown-menu').append(
+                  $('<li>').append(
+                    $('<div>').addClass('input-group input-group-sm').append(
+                      $('<input>', {type: 'text', placeholder: 'Enter Path'}).addClass('form-control')
+                    ).append(
+                      $('<span>').addClass('input-group-btn').append(
+                        $('<button>', {type: 'button'}).addClass('btn btn-default').prop('disabled', true).append(
+                          $('<span>').addClass('glyphicon glyphicon-ok-sign')    
+                        )
+                      )
+                    )
+                  )
+                )
+              )
             ).append(
               $('<div>').addClass('sF-refresh btn-group btn-group-sm').append(
                 $('<button>', {id: 'sF-btn-refresh'}).addClass('btn btn-default').append(
@@ -929,6 +967,40 @@ var shinyFiles = (function() {
         
         $(modal).trigger('fileSort', [$(this).parent().find('.selected a').text(), $(this).find('a').attr('class')])
       })
+
+		// Text Choice
+        modal.find('.sF-textChoice').on('click', function() {
+          var directory = getCurrentDirectory(modal);
+          var disabled = $(this).find('button').prop('disabled')
+          if(!disabled) {
+            $(this).toggleClass('open')
+              .find('button.sF-btn-textChoice').toggleClass('active');
+            if($(this).hasClass('open')) {
+              $(this).find('input').val(directory.root + directory.path.join("/")).focus();
+              $(this).find('.input-group-btn>button').prop('disabled', true);
+            }
+          }
+          return false;
+		})
+		modal.find('.sF-textChoice input').on('keyup', function(e) {
+          var disabled = $(this).val() == '';
+          $(this).parent().find('button').prop('disabled', disabled);
+          if(e.keyCode == 13) {
+             e.stopPropagation();
+             setPathFromTextInput($(this).val(), modal);
+          } else if(e.keyCode == 27) {
+            var parent = $(this).closest('.sF-textChoice');
+            parent.toggleClass('open', false)
+              .find('button.sF-btn-textChoice').toggleClass('active', true);
+          }
+        })
+		modal.find('.sF-textChoice ul').on('click', function() {
+          return false;
+        })
+        modal.find('.sF-textChoice ul button').on('click', function() {
+          var name = $(this).closest('.input-group').find('input').val();
+          setPathFromTextInput(name,modal);
+        })
         
     // Breadcrump and volume navigation
     modal.find('.sF-breadcrumps').on('change', function() {
@@ -981,6 +1053,11 @@ var shinyFiles = (function() {
     
     // Ready to enter
     setTimeout(function() {
+      if($('#shiny-modal').length ==0){
+        modal.detach().appendTo('body')
+      }else{
+        modal.detach().appendTo('#shiny-modal')
+      }
       modal.addClass('in');
       backdrop.addClass('in');
     }, 1);
@@ -989,6 +1066,7 @@ var shinyFiles = (function() {
   };
 
   var populateFileChooser = function(element, data, forceUpdate) {
+
     var modal = $(element).data('modal');
       
     $(element).data('dataCache', data);
@@ -1057,7 +1135,7 @@ var shinyFiles = (function() {
         var d = data.files[i];
         
         modal.find('.sF-fileList').append(
-          $('<div>').toggleClass('sF-file', !d.isDir).toggleClass('sF-directory', d.isDir).append(
+          $('<div>').toggleClass('sF-file', !d.isDir).toggleClass('selected',d.name==data.selectedFile).toggleClass('sF-directory', d.isDir).append(
             $('<div>').addClass('sF-file-icon').addClass('sF-filetype-'+d.extension)
           ).append(
             $('<div>').addClass('sF-file-name').append(
@@ -1086,12 +1164,13 @@ var shinyFiles = (function() {
         }).remove();
       };
       
+      
       if (Object.keys(newFiles).length === 0) {
         for (i in newFiles) {
           var d = newFiles[i];
           
           modal.find('.sF-fileList').append(
-            $('<div>').toggleClass('sF-file', !d.isDir).toggleClass('sF-directory', d.isDir).append(
+            $('<div>').toggleClass('sF-file', !d.isDir).toggleClass('selected',d.name==data.selectedFile).toggleClass('sF-directory', d.isDir).append(
               $('<div>').addClass('sF-file-icon').addClass('sF-filetype-'+d.extension)
             ).append(
               $('<div>').addClass('sF-file-name').append(
@@ -1113,7 +1192,11 @@ var shinyFiles = (function() {
     
     if($(element).hasClass('shinySave')) {
       setPermission(modal, data.writable);
+      if(data.selectedFile != ""){
+        setFilename(modal,data.selectedFile)
+      }
     }
+    setExists(modal, data.exist);
     toggleSelectButton(modal);
     
     modal.data('currentData', data);
@@ -1276,7 +1359,7 @@ var shinyFiles = (function() {
   
   // Creating modal
   var modal = $('<div>', {id: $(button).attr('id')+'-modal'}).addClass('sF-modalContainer modal fade').css('display', 'block').append(
-      $('<div>').addClass('sF-modal modal-dialog').append(
+      $('<div>').addClass('sF-modal modal-dialog modal-lg').append(
         $('<div>').addClass('modal-content').append(
           $('<div>').addClass('modal-header').append(
             $('<button>', {html: '&times;', type: 'button'}).addClass('close')
@@ -1347,6 +1430,26 @@ var shinyFiles = (function() {
                   )
                 )
               )
+            ).append( 
+              $('<div>').addClass('sF-textChoice btn-group input-group-btn btn-group-sm').append(
+                $('<button>', {id: 'sF-btn-textChoice', text: ''}).addClass('btn btn-default dropdown-toggle').prepend(
+                  $('<span>').addClass('glyphicon glyphicon-pencil')
+                )
+              ).append(
+                $('<ul>').addClass('dropdown-menu').append(
+                  $('<li>').append(
+                    $('<div>').addClass('input-group input-group-sm').append(
+                      $('<input>', {type: 'text', placeholder: 'Enter Path'}).addClass('form-control')
+                    ).append(
+                      $('<span>').addClass('input-group-btn').append(
+                        $('<button>', {type: 'button'}).addClass('btn btn-default').prop('disabled', true).append(
+                          $('<span>').addClass('glyphicon glyphicon-ok-sign')    
+                        )
+                      )
+                    )
+                  )
+                )
+              )
             ).append(
               $('<div>').addClass('sF-refresh btn-group btn-group-sm').append(
                 $('<button>', {id: 'sF-btn-refresh'}).addClass('btn btn-default').append(
@@ -1474,6 +1577,41 @@ var shinyFiles = (function() {
         }
         return false;
       })
+
+        //Folder Text Selection
+        modal.find('.sF-textChoice').on('click', function() {
+          var directory = getCurrentDirectory(modal);
+          var disabled = $(this).find('button').prop('disabled')
+          if(!disabled) {
+            $(this).toggleClass('open')
+              .find('button.sF-btn-textChoice').toggleClass('active');
+            if($(this).hasClass('open')) {
+              $(this).find('input').val(directory.root + directory.path.join("/")).focus();
+              $(this).find('.input-group-btn>button').prop('disabled', true);
+            }
+          }
+          return false;
+		})
+		modal.find('.sF-textChoice input').on('keyup', function(e) {
+          var disabled = $(this).val() == '';
+          $(this).parent().find('button').prop('disabled', disabled);
+          if(e.keyCode == 13) {
+            e.stopPropagation();
+            setPathFromTextInput($(this).val(), modal);
+          } else if(e.keyCode == 27) {
+            var parent = $(this).closest('.sF-textChoice');
+            parent.toggleClass('open', false)
+              .find('button.sF-btn-textChoice').toggleClass('active', true);
+          }
+        })
+		modal.find('.sF-textChoice ul').on('click', function() {
+          return false;
+        })
+        modal.find('.sF-textChoice ul button').on('click', function() {
+          var name = $(this).closest('.input-group').find('input').val();
+          setPathFromTextInput(name,modal);
+        })
+        
         
     // Folder creation
     modal.find('.sF-newDir').on('click', function() {
@@ -1546,6 +1684,11 @@ var shinyFiles = (function() {
     
     // Ready to enter
     setTimeout(function() {
+      if($('#shiny-modal').length ==0){
+        modal.detach().appendTo('body')
+      }else{
+        modal.detach().appendTo('#shiny-modal')
+      }
       modal.addClass('in');
       backdrop.addClass('in');
     }, 1);
@@ -1618,7 +1761,7 @@ var shinyFiles = (function() {
   var evalFilename = function(modal) {
     var parent = modal.find('.sF-filenaming');
     var name = getFilename(modal).name;
-    modal.find('#sF-selectButton').prop('disabled', name == '' || modal.find('.sF-permission').length != 0);
+    modal.find('#sF-selectButton').prop('disabled', name == '' || modal.find('.sF-warning').length != 0);
     if(nameExist(modal, name)) {
       parent.toggleClass('has-feedback', true)
         .toggleClass('has-warning', true);
@@ -1690,7 +1833,7 @@ var shinyFiles = (function() {
           
     // Create the dialog
     var modal = $('<div>', {id: $(button).attr('id')+'-modal'}).addClass('sF-modalContainer modal fade').css('display', 'block'  ).append(
-      $('<div>').addClass('sF-modal modal-dialog').append(
+      $('<div>').addClass('sF-modal modal-dialog modal-lg').append(
         $('<div>').addClass('modal-content').append(
           $('<div>').addClass('modal-header').append(
             $('<button>', {html: '&times;', type: 'button'}).addClass('close')
@@ -1753,6 +1896,26 @@ var shinyFiles = (function() {
                   )
                 )
               )
+            ).append( 
+              $('<div>').addClass('sF-textChoice btn-group input-group-btn btn-group-sm').append(
+                $('<button>', {id: 'sF-btn-textChoice', text: ''}).addClass('btn btn-default dropdown-toggle').prepend(
+                  $('<span>').addClass('glyphicon glyphicon-pencil')
+                )
+              ).append(
+                $('<ul>').addClass('dropdown-menu').append(
+                  $('<li>').append(
+                    $('<div>').addClass('input-group input-group-sm').append(
+                      $('<input>', {type: 'text', placeholder: 'Enter Path'}).addClass('form-control')
+                    ).append(
+                      $('<span>').addClass('input-group-btn').append(
+                        $('<button>', {type: 'button'}).addClass('btn btn-default').prop('disabled', true).append(
+                          $('<span>').addClass('glyphicon glyphicon-ok-sign')    
+                        )
+                      )
+                    )
+                  )
+                )
+              )
             ).append(
               $('<div>').addClass('sF-refresh btn-group btn-group-sm').append(
                 $('<button>', {id: 'sF-btn-refresh'}).addClass('btn btn-default').append(
@@ -1809,6 +1972,128 @@ var shinyFiles = (function() {
     modal.find('.sF-responseButtons #sF-selectButton').on('click', function() {
       selectFiles(button, modal);
     })
+
+      //Folder Text Selection
+      var getCurrentDirectoryFolderSelection = function(modal){
+        var path = getPath($(modal).find('.sF-dirList .selected'));
+        $(button).data('directory', path)
+          .trigger('selection', path);
+        var data = {
+          path: path,
+          root: $(modal).data('currentData').selectedRoot
+        };
+        return data;
+      }
+		
+      modal.find('.sF-textChoice').on('click', function() {
+        var directory = getCurrentDirectoryFolderSelection(modal);
+        var disabled = $(this).find('button').prop('disabled')
+        if(!disabled) {
+          $(this).toggleClass('open')
+            .find('button.sF-btn-textChoice').toggleClass('active');
+          if($(this).hasClass('open')) {
+            $(this).find('input').val(directory.root + directory.path.join("/")).focus();
+            $(this).find('.input-group-btn>button').prop('disabled', true);
+          }
+        }
+        return false;
+      })
+      var setPathFromTextInputFolderSelection = function(path, modal) {
+        if(path != '') {
+          var currentDir = getCurrentDirectoryFolderSelection(modal);
+          var date = new Date();
+          var data = {
+              name: path,
+              path: currentDir.path,
+              root: currentDir.root,
+              id: date.getTime()
+          }
+          
+          var breadcrumps = modal.find('.sF-breadcrumps')
+          var volumes = breadcrumps.find('optgroup option')
+         
+          //Set a new root, if there is a match in the text path
+          var foundRoot = false;
+          $(volumes).each(function(i,volume) {
+            if(path.startsWith($(volume).val())){
+              foundRoot = true;
+              if(data.root != $(volume).val()){
+                data.root = $(volume).val();
+                modal.data('currentData').selectedRoot = data.root;
+              }
+              data.path = path.substr(data.root.length).split(/[/\\]/);
+            }
+          })
+
+          
+          if(!foundRoot){
+            var searchPattern = new RegExp(/^[/\\]/);
+            if(searchPattern.test(path)){ //From the root
+              data.path = path.split(/[/\\]/);
+            }else{ //relative path
+              data.path.push(path.split(/[/\\]/));
+            }
+          }
+          
+          //deselect selected elements
+          $('.sF-dirList .selected').toggleClass('selected');
+          parent = $(".sF-dirList .sF-directory :contains(" + data.root + ")").closest(".sF-directory");
+
+          //traverse the new path and expand directories
+          if(!parent.hasClass('empty')) {
+            modal.data('currentData').tree = new Object();
+            tree = modal.data('currentData').tree;
+            tree.name = "";
+            tree.expanded = false;
+            tree.empty = true;
+            tree.children = new Array(0);
+            var tpath = JSON.parse(JSON.stringify(data.path))
+            tpath.shift();
+            for(name of tpath){
+              tree.expanded = true;
+              tree.empty = false;
+              tree.children = new Array(new Object);
+              tree = tree.children[0];
+              tree.name = name
+              tree.expanded = false;
+              tree.empty = true;
+              tree.children = new Array(0);
+            }
+          }
+          
+          //select the final path element
+          parent.toggleClass('selected');
+
+          $(modal).find('.sF-textChoice').toggleClass('open', false)
+            .find('.sF-btn-textChoice').toggleClass('active', false);
+
+          modal.data('currentData').contentPath = data.path;
+              
+          Shiny.onInputChange($(button).attr('id')+'-modal', modal.data('currentData'));
+                      
+        }
+        return false;
+      };  
+        
+      modal.find('.sF-textChoice input').on('keyup', function(e) {
+        var disabled = $(this).val() == '';
+        $(this).parent().find('button').prop('disabled', disabled);
+        if(e.keyCode == 13) {
+          e.stopPropagation();
+          setPathFromTextInputFolderSelection($(this).val(), modal);
+        } else if(e.keyCode == 27) {
+          var parent = $(this).closest('.sF-textChoice');
+          parent.toggleClass('open', false)
+            .find('button.sF-btn-textChoice').toggleClass('active', true);
+        }
+      })
+      modal.find('.sF-textChoice ul').on('click', function() {
+        return false;
+      })
+      modal.find('.sF-textChoice ul button').on('click', function() {
+        var name = $(this).closest('.input-group').find('input').val();
+        setPathFromTextInputFolderSelection(name,modal);
+      })
           
     // Create dir
     modal.find('.sF-newDir').on('click', function() {
@@ -1902,6 +2187,11 @@ var shinyFiles = (function() {
           
     // Ready to enter
     setTimeout(function() {
+      if($('#shiny-modal').length ==0){
+        modal.detach().appendTo('body')
+      }else{
+        modal.detach().appendTo('#shiny-modal')
+      }
       modal.addClass('in');
       backdrop.addClass('in');
     }, 1);
@@ -1949,17 +2239,14 @@ var shinyFiles = (function() {
     } else {
       dirContent.toggleClass('message', false)
       data.content.filename.forEach(function(file, i) {
-        var mTime = data.content.mtime[i].split('-');
-        var cTime = data.content.ctime[i].split('-');
-        var aTime = data.content.atime[i].split('-');
         var d = {
           name: file,
           isDir: data.content.isdir[i],
           extension: data.content.extension[i],
           size: data.content.size[i],
-          mTime: new Date(mTime[0], mTime[1]-1, mTime[2], mTime[3], mTime[4]),
-          cTime: new Date(cTime[0], cTime[1]-1, cTime[2], cTime[3], cTime[4]),
-          aTime: new Date(aTime[0], aTime[1]-1, aTime[2], aTime[3], aTime[4])
+          mTime: new Date(data.content.mtime[i]),
+          cTime: new Date(data.content.ctime[i]),
+          aTime: new Date(data.content.atime[i])
         };
         dirContent.append(
           $('<div>').toggleClass('sF-file', !d.isDir).toggleClass('sF-directory', d.isDir).append(
@@ -1981,6 +2268,7 @@ var shinyFiles = (function() {
     
     toggleSelectButton(modal);
     setPermission(modal, data.writable);
+    setExists(modal,data.exist);
     
     modal.data('currentData', data);
     $(modal).trigger('change');
@@ -2173,6 +2461,51 @@ var shinyFiles = (function() {
         .find('.sF-btn-newDir').toggleClass('active', false);
     }
   };
+
+  var setPathFromTextInput = function(path, modal) {
+    if(path != '') {
+      var button = $($(modal).data('button'));
+      var currentDir = getCurrentDirectory(modal);
+      var date = new Date();
+      var data = {
+          name: path,
+          path: currentDir.path,
+          root: currentDir.root,
+          id: date.getTime()
+      }
+      
+      var breadcrumps = modal.find('.sF-breadcrumps')
+      var volumes = breadcrumps.find('optgroup option')
+     
+      //Set a new root, if there is a match in the text path
+      var foundRoot = false;
+      $(volumes).each(function(i,volume) {
+        if(path.startsWith($(volume).val())){
+          data.root = $(volume).val();
+          data.path = path.substr(data.root.length).split(/[/\\]/)
+          foundRoot = true;
+        }
+      })
+      if(!foundRoot){
+        var searchPattern = new RegExp(/^[/\\]/);
+        if(searchPattern.test(path)){ //From the root
+            data.path = path.split(/[/\\]/);
+        }else{ //relative path
+            data.path.push(path.split(/[/\\]/));
+        }
+      }
+      
+      $(modal).find('.sF-textChoice').toggleClass('open', false)
+        .find('.sF-btn-textChoice').toggleClass('active', false);
+  
+  
+      modal.data('currentData').contentPath = data.path;
+      Shiny.onInputChange($(button).attr('id')+'-modal', data);
+      $(button).data('back').push(currentDir);
+      $(button).data('forward', []);
+    }
+    return false;
+  };
     
   var getPath = function(element) {
     if(element.hasClass('.sF-directory')) {
